/**************************************************************************/
/*  parchis.c Version 4.0                                                 */
/*  Copyright (C) 1992, 1999, 2007 Mariano Alvarez Fernandez              */
/*  Lea en el fichero LEEME.TXT los detalles                              */
/*  See README.TXT for details                                            */
/**************************************************************************/
/*  Juega al Parchis                                                      */
/**************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <time.h>     // time
#include <mgrx.h>
#include "gfaz.h"
#include "parchit.h"
#include "parchig.h"
#include "idiomas.h"

/* Rutinas internas */

int juega_persona( struct Tablero *tbl, int i );
int juega_maquina( struct Tablero *tbl, int i );

/**************************************************************************/

int main()
{
  struct Tablero tbl;
  int tjamar = PERSONA, tjazul = NOJUEGA, tjrojo = MAQUINA, tjverd = NOJUEGA;
  int ret;

  srandom( time( NULL ) );
  gfaz_ini();
  GrSetWindowTitle( "Parchis" );
  inicia_tablero( &tbl );
  inicia_graficos();
  pantalla_inicial();
  if( (idioma = elige_idioma( 215,180 )) < 0 ) termina();
  if( quien_juega( 177,30,&tjamar,&tjazul,&tjrojo,&tjverd ) != 0 ) termina();
  inicia_jugadores( &tbl,tjamar,tjazul,tjrojo,tjverd );
  dibuja_pantalla();
  situa_fichasg( &tbl );

  while( 1 ){
    if( tipo_jugador( &tbl,tbl.turno ) == PERSONA )
      ret = juega_persona( &tbl,tbl.turno );
    else if( tipo_jugador( &tbl,tbl.turno ) == MAQUINA )
      ret = juega_maquina( &tbl,tbl.turno );
    if( hay_orden( &tbl ) == -1 ) ret = 1;
    if( ret == 0 )
      tbl.turno = (tbl.turno == VERD) ? 0 : tbl.turno + 1;
    if( tbl.num_jugadores == tbl.siguiente_final )
      partida_acabada( &tbl );
    ret = 0;
    }
}

/**************************************************************************/

int juega_persona( struct Tablero *tbl, int i )
{
  struct Tablero tbl2;
  struct JugadasPosibles jpo;
  struct Jugada jugada;
  int nj, vdado, jfin;
  int nfmovida, num6, vdador;

  num6 = 0;
  do{
    pinta_dado( 0 );
    pinta_tirador( i, ST_TIRAS );
    if( espera_orden( tbl ) != 0 ) return 1;
    vdador = vdado = dadocorr( tbl,i );
/*
    sound( 100 ); GrSleep( 100 ); nosound();
    c = getch() - '0';
    if( c > 0 && c < 8 ) vdador = vdado = c;
*/
    if( vdado == 6 || vdado == 7 ) num6++;
    if( num6 == 3 && tbl->jugador[i].ficha[nfmovida].lugar == CORREDOR ){
      pinta_dado( vdado );
      pinta_tirador( i,ST_ACASA );
      ruido_a_casa();
      if( espera_orden( tbl ) != 0 ) return 1;
      tbl2 = *tbl;
      tres_seises( &tbl2,i,nfmovida,&jugada );
      mueve_fichasg( tbl,&tbl2,&jugada,0 );
      *tbl = tbl2;
      }
    else{
      while( vdado > 0 ){
        pinta_dado( vdado );
        nj = genera_jugadas_posibles( tbl,&jpo,vdado,i );
        if( nj > 0 ){
          if( jpo.jugada[0].anal.entra ){
            jugada = jpo.jugada[0];
            pinta_tirador( i,ST_SALES );
            if( espera_orden( tbl ) != 0 ) return 1;
            }
          else{
            pinta_tirador( i,ST_MUEVES );
            if( elige_jugadag( tbl,&jpo,&jugada ) != 0 ) return 1;
            }
          tbl2 = *tbl;
          vdado = realiza_jugada( &tbl2,&jugada );
          jfin = ( vdado == -1 ) ? 1 : 0;
          mueve_fichasg( tbl,&tbl2,&jugada,jfin );
          *tbl = tbl2;
          nfmovida = jugada.nficha;
          if( jfin ) vdador = 0;  /* para que no tire si ya llego */
          }
        else{
          pinta_tirador( i,ST_PASAS );
          if( espera_orden( tbl ) != 0 ) return 1;
          vdado = 0;
          }
        }
      }
    } while( (vdador == 6 || vdador == 7 ) && num6 < 3 );

  return 0;
}

/**************************************************************************/

int juega_maquina( struct Tablero *tbl, int i )
{
  struct Tablero tbl2;
  struct JugadasPosibles jpo;
  struct Jugada jugada;
  int nj, vdado, jfin;
  int nfmovida, num6, vdador;
  int valores[4];

  num6 = 0;
  do{
    pinta_dado( 0 );
    pinta_tirador( i, ST_TIRO );
    vdador = vdado = dadocorr( tbl,i );
    if( vdado == 6 || vdado == 7 ) num6++;
    if( num6 == 3 && tbl->jugador[i].ficha[nfmovida].lugar == CORREDOR ){
      pinta_dado( vdado );
      pinta_tirador( i,ST_ACASA );
      ruido_a_casa();
      GrSleep( 1000 );
      tbl2 = *tbl;
      tres_seises( &tbl2,i,nfmovida,&jugada );
      mueve_fichasg( tbl,&tbl2,&jugada,0 );
      *tbl = tbl2;
      }
    else{
      while( vdado > 0 ){
        pinta_dado( vdado );
        if( hay_orden( tbl ) == -1 ) return 1;
        nj = genera_jugadas_posibles( tbl,&jpo,vdado,i );
        if( nj > 0 ){
          if( jpo.jugada[0].anal.entra ){
            jugada = jpo.jugada[0];
            pinta_tirador( i,ST_SALGO );
            GrSleep( 1000 );
            }
          else{
            pinta_tirador( i,ST_MUEVO );
            elige_jugadam( tbl,&jpo,&jugada,valores );
#ifdef DEPURAR
            pinta_anal_global( &jpo,valores );
#endif
            pinta_jugadam( tbl,&jugada );
            }
          tbl2 = *tbl;
          vdado = realiza_jugada( &tbl2,&jugada );
          jfin = ( vdado == -1 ) ? 1 : 0;
          mueve_fichasg( tbl,&tbl2,&jugada,jfin );
          *tbl = tbl2;
          nfmovida = jugada.nficha;
          if( jfin ) vdador = 0;  /* para que no tire si ya llego */
          }
        else{
          pinta_tirador( i,ST_PASO );
          GrSleep( 1000 );
          vdado = 0;
          }
        }
      }
    } while( (vdador == 6 || vdador == 7 ) && num6 < 3 );

  return 0;
}

